###################################################################
###### From raw p-values to adjusted p-values and q-values ########
###################################################################

#' Adjusted p-values and q-values  
#' @description 
#' From a raw pvalues the adjusted p-values and q-values 
#' are calculated 
#' @param rawp Raw p-values
#' @param correction Correction method
#' @export
#' @return 
#' A data frame con adjp, adjusted p-values and q-values
#' 
rawp2adjpq = function(rawp,correction){
  ##Adjusted values
  rawp[rawp <= 0] = 0
  rawp[rawp >=1] = 1
  adjp = stats::p.adjust(rawp,method = correction) 
  ## Calculate the Storey q-value
  qval = qvalue::qvalue(rawp)$qvalues 
  data.frame(adjp = adjp, qval = qval)
}

#----  Marginal differential expression with asymptotic distribution
#----  MICROARRAYS

#' Tests per row (feature) for marginal differential expression 
#' @description 
#' For a given expression matrix \code{x} and a given (two-level)
#' factor \code{y} the two sample \code{test} is applied. The expression
#' matrix probably is obtained as \code{x = exprs(gse)} or
#' \code{x = assay(se)} where \code{gse} is a \code{ExpressionSet} and 
#' \code{se} is a \code{RangedSummarizedExperiment}
#' @param x Matrix expression
#' @param y Two-level factor 
#' @param test Function with the statistical test (c("rowt","rowtmod",
#' "edgercommon","edgertagwise"))
#' @param correction Correction method for multiple testing
#' 
#' @export
#' @return 
#' A data frame with columns named \code{statistic} (statistic of the test),
#' \code{rawp} (original p-values), \code{adjp} (adjusted p-values) and 
#' \code{qval} (q-values)
rowtest = function(x,y,test,
                   correction=c("BH","bonferroni","holm","hochberg", "hommel",
                                "BH", "BY","none")){
  correction = match.arg(correction)
  tp = test(x,y)
  names(tp) = c("statistic","rawp")
  output = data.frame(tp,rawp2adjpq(tp[,"rawp"],correction))
  rownames(output) = rownames(x)
  output
}

###############################################################
############## Continuous expression (microarrays, ...) #######
###############################################################

#' t-tests per row ...
#' @description
#' We use the functions \link[genefilter]{rowttests}
#' @param x Matrix with expression values (exprs() or assay())
#' @param y Factor giving the classification of the sample into two groups
#' @return
#' A data.frame with columns:
#' \describe{
#'  \item{statistic}{The two-sample t-statistic}
#'  \item{rawp}{The raw p-values of the two sample t-test}
#' }
#' @examples
#' pacman::p_load("Biobase")
#' data(gse1397,package="tamidata")
#' result = rowt(x=exprs(gse1397),y=pData(gse1397)[,"type"])
#' @keywords Marginal diferential expression
#' 
#' @export
#'
#' @seealso 
#' \link[genefilter]{rowttests}
#' 
rowt = function(x,y){
  if(!is.matrix(x)) message("x must be a matrix \n")
  if(!is.factor(y)) message("y must be a factor \n")
  if(nlevels(y) != 2) message("y must have two levels \n")
  ## Calculate t-statistic and p-value
  tt = genefilter::rowttests(x,y)
  statistic = tt[,"statistic"]
  rawp = tt[,"p.value"]
  data.frame(statistic,rawp)
}

#' Moderated t-test per gene using limma
#' @description
#' Moderated t-test per gene using Limma package
#' @param x Matrix with expression values (exprs() or assay())
#' @param y Experimental factor
#' @return
#' A data.frame with columns:
#' \describe{
#'  \item{statistic}{The two-sample t-statistic}
#'  \item{rawp}{The raw p-values of the two sample t-test}
#'  }
#' @examples
#' pacman::p_load(Biobase)
#' data(gse1397,package="tamidata")
#' result = rowtmod(x=exprs(gse1397),y = pData(gse1397)[,"type"])
#' @keywords Marginal diferential expression
#' @export
#'
#' @seealso \link[qvalue]{qvalue}
#'  
rowtmod = function(x,y){
  if(!is.matrix(x)) message("x must be a matrix \n")
  if(!is.factor(y)) message("y must be a factor \n")
  if(nlevels(y) != 2) message("y must have two levels \n")
  design = stats::model.matrix(~ 0 + y)
  colnames(design) = c("Group1","Group2")
  fit = limma::lmFit(x,design)
  cont.matrix = limma::makeContrasts(dif= "Group1 - Group2",levels = design)
  fit2 = limma::contrasts.fit(fit,cont.matrix)
  fit2 = limma::eBayes(fit2)
  out = limma::topTable(fit2,coef=1,number=nrow(x),sort.by = "none")
  statistic = out[,"t"]
  rawp = out[,"P.Value"]
  data.frame(statistic,rawp)
}

##################################################
############## RNA-seq ###########################
##################################################


#' edgeR with common dispersion
#' 
#' @description
#' Given a count matrix \code{x}  and an experimental factor
#' \code{y} with two levels the t-statistic (logFC) or p-valor 
#' of the comparison between both groups is calculated. 
#' The procedure was proposed in
#' \url{https://doi.org/10.1093/biostatistics/kxm030} 
#' and  \url{https://doi.org/10.1093/bioinformatics/btm453}
#' and it is implemented in the package edgeR.
#' @param x Matrix of counts
#' @param y Experimental factor (with two levels)
#' @export
#' @return
#' A data.frame with \code{statistic} (logFC)
#' and \code{rawp} p-values of exact test using a common 
#' dispersion. Its length is equal to the number of rows
#' of the \code{counts0} matrix
edgercommon = function(x,y){
  dge = edgeR::DGEList(counts=x,group=y)
  dge.c = edgeR::estimateCommonDisp(dge) 
  et.c = edgeR::exactTest(dge.c)
  statistic = et.c$table[,"logFC"]
  rawp = et.c$table[,"PValue"]
  data.frame(statistic,rawp)
}

#' edgeR with tagwise dispersion  
#' @description
#' Given a count matrix \code{x}  and an experimental factor
#' \code{y} with two levels the t-statistic (logFC) or p-valor of the 
#' comparison between both groups is calculated. 
#' The procedure was proposed in
#' \url{https://doi.org/10.1093/biostatistics/kxm030} 
#' and  \url{https://doi.org/10.1093/bioinformatics/btm453}
#' and it is implemented in the package edgeR.
#' @param x Matrix of counts
#' @param y Experimental factor (with two levels)
#' @export
#' @return
#' A data.frame with \code{statistic} (logFC)
#' and \code{rawp} p-values of exact test using a common 
#' dispersion. Its length is equal to the number of rows
#' of the \code{counts0} matrix.
#' 
edgertagwise = function(x,y){
  dge = edgeR::DGEList(counts=x,group=y)
  dge.c = edgeR::estimateCommonDisp(dge) 
  dge.t = edgeR::estimateTagwiseDisp(dge.c) 
  et.t = edgeR::exactTest(dge.t)
  statistic  = et.t$table[,"logFC"]
  rawp  = et.t$table[,"PValue"]
  data.frame(statistic,rawp)
}

############################################################
################ GLOBAL FUNCTIONS ##########################
############################################################  

#' Differential expression for microarrays and RNA-seq data
#' @description 
#' Differential expression for microarrays and RNA-seq data
#' @param x \code{ExpressionSet} or \code{RangedSummarizedExperiment}
#' @param y Name of a two-level factor
#' @param test Testing procedure for marginal differential expression.
#' It should return a \code{data.frame} with two columns named
#' \code{statistic} and \code{rawp}. Some possible \code{test}'s are
#' \link{rowt},\link{rowtmod},\link{edgercommon},\link{edgertagwise}
#' @param correction Type of correction for multiple testing 
#' used by  \code{stats::p.adjust}
#' @param fdr False discovery rate
#' @param foutput File for html report
#' (\code{stats::p.adjust})
#' @export
dema = function(x,y,test,
                correction=c("BH","bonferroni","holm","hochberg", "hommel",
                             "BH", "BY","none"),
                fdr= 0.01,
                foutput = "dema_output"){
  xed = convert(x,y)
  if(type(xed) == "ExpressionSet"){
    if(ncol(Biobase::fData(x))==0) Biobase::fData(x) = Biobase::featureNames(x)
    xa = Biobase::fData(x)
  }
  if(type(xed) == "RangedSummarizedExperiment"){
    if(ncol(SummarizedExperiment::rowData(x))==0) 
      SummarizedExperiment::rowData(x) = rownames(x)
    xa = data.frame(SummarizedExperiment::rowData(x))
  }
  xt = rowtest(x=exprm(xed),y=groups(xed),test=test,correction=correction)
  DifferentialExpressionOutput(GeneData = xa,GeneStat = xt,
                                      foutput=foutput,fdr=fdr,
                                      exprm=exprm(xed),groups=groups(xed))
}


