
#' Random gene expression
#' @description 
#' Function to generate all models of gene expression
#' @param generator Function generating the random matrix
#' @param theta List with the parameters of the stochastic models used
#' @return
#' An expression matrix  
#' 
#' @export
#' 
rgenexprs = function(generator,theta) generator(theta)




#' Beta and logit transformation   
#' @description 
#' It generates random expression for `theta$n0` genes under two conditions,
#' `theta$n1` and `theta$n2` samples for the two conditions. 
#' Each gene has a lognormal distribution.
#' We are going to described the parameters of the normal values generated.
#' Each normal value generated have a different mean and standard deviation 
#' per sample and condition. 
#' The means for the first condition goes from `theta$m1` to `theta$m2` and the 
#' values are calculated as the quantiles of a transformed beta distribution, 
#' `theta$m1 + theta$m2* qbeta(seq(0,1,length.out = theta$n0),
#' shape1 = theta$alpha,shape2=theta$beta)`.
#' The standard deviations for the first condition are generated as 
#' `theta$sigma * rchisq(theta$n0,df=theta$df0)` i.e. proportional to a 
#' chisquare distribution. 
#' The means of the second conditions are obtained using a logit transformation
#' from the first previously calculated means for the first condition. 
#' The standard deviations are generated as for the first condition where
#' the factor is now `theta$sigmafac*theta$sigma`.
#' 
#' @param theta List with parameters
#' @export
#' @keywords Random gene expression  
#' 
logbetalogit = function(theta){
  mu1 = theta$m1 + theta$m2* stats::qbeta(seq(0,1,length.out = theta$n0),
                                   shape1 = theta$alpha,shape2=theta$beta)
  p1 = exp(-theta$m1) /(1+ exp(-theta$m1))
  p2 = exp(theta$m2-theta$m1) /(1+ exp(theta$m2-theta$m1))
  p = seq(p1,p2,length.out = theta$n0)
  mu2 = mu1 + log(p/(1-p))
  sd1 = theta$sigma * stats::rchisq(theta$n0,df=theta$sigmadf) 
  sd2 = theta$sigmafac*theta$sigma * stats::rchisq(theta$n0,df=theta$sigmadf)
  exprset0 = 
    cbind(replicate(theta$n1,unlist(Map(function(mu,sigma) 
      stats::rnorm(1,mean=mu,sd=sigma),mu1,sd1))),
      replicate(theta$n2,unlist(Map(function(mu,sigma) 
        stats::rnorm(1,mean=mu,sd=sigma),mu2,sd2))))
  exp(exprset0)
}

#' Normal data using normal inverse chisquare
#' @description 
#' It generates an expression matrix with `theta$n0` rows (features)
#' for two conditions or sample groups of `theta$n1` and `theta$n2`.
#' The first sample group occupied the first `theta$n1` columns and the
#' second sample group occupied the last `theta$n2` columns.
#' For each feature in the first group the random expression follows a normal
#' distribution with mean and standard deviation generated as a normal
#' inverse chisquare. 
#' We chose randomly a `theta$p_up` proportion of up-regulated genes
#' and a `theta$p_down` proportion of down-regulated genes. The rest of genes
#' has no differential expression.
#' The up-regulated genes has a normal distribution where the mean are the 
#' original means for the first condition plus a normal value with mean 
#' `theta$mu_up` and standard deviation `theta$sigma_up`. 
#' The down-regulated genes has a normal distribution where the mean are the 
#' original
#' means for the first condition plus a normal value with mean `theta$mu_down`
#' and standard deviation `theta$sigma_down`. 
#' @param theta List with the parameters
#' @export

nic = function(theta){
  sigma1 = (theta$nu0*theta$sigma0^2) / stats::rchisq(theta$n0,df=theta$nu0) 
  mu1 = unlist(lapply(sigma1, function(x) 
    stats::rnorm(1,mean=theta$mu0,sd=x/sqrt(theta$kappa0))))
  sigup = sample(1:theta$n0,round(theta$n0*theta$p_up))
  sigdown = sample(setdiff(1:theta$n0,theta$sigup),round(theta$n0*theta$p_down))
  mu2 = mu1
  mu2[sigup] = mu1[sigup] + 
    stats::rnorm(length(sigup),mean=theta$mu_up,sd=theta$sigma_up)
  mu2[sigdown] = mu1[sigdown] + 
    stats::rnorm(length(sigdown),mean=theta$mu_down,sd=theta$sigma_down)
  sigma2 = sigma1
  cbind(replicate(theta$n1,unlist(Map(function(x,y) stats::rnorm(1,mean=x,sd=y),
                                      mu1,sigma1))),
        replicate(theta$n2,unlist(Map(function(x,y) stats::rnorm(1,mean=x,sd=y),
                                      mu2,sigma2))))
}

## GENERATIONS OF RANDOM COUNTS (RNA-seq,...)

#' @importFrom  stats rpois 
NULL

#' Random counts with Poisson distribution
#' @description 
#' A gene expression matrix of counts with \code{N} features and two 
#' independent conditions, the first \code{n1} columns corresponds to the first
#' condition and the last \code{n2} corresponds to the second condition.  
#' X[i,j] follows a Poisson distribution with mean \code{lambda_j} such
#' that \code{lambda_j = exp(beta[0])} for \code{j = 1:n1} and
#' \code{lambda_j = exp(beta[0] + beta[1])} for \code{j = (n1+1):(n1+n2)}
#' @param n1 Number of samples of the first condition (control)
#' @param n2 Number of samples of the second condition (treatment)
#' @param N Number of features (rows in the expression matrix)
#' @param beta Vector of coefficients
#' 
#' @export
#' 
#' @keywords 
#' Generation of expression matrix, random counts
#' 
poiseq = function(n1,n2,N,beta){
  xi = cbind(rep(1,n1+n2),rep(0:1,c(n1,n2)))
  mu = exp(apply(xi,1,function(u) sum(u * beta)))
  sapply(mu,function(lambda0) stats::rpois(N,lambda=lambda0))
}

#' @importFrom  MASS rnegbin 
NULL


#' Random paired counts: model 1
#' @description 
#' A gene expression matrix of counts with \code{N} features and
#' \code{length(n)} pairs of samples.
#' The first pair corresponds to the two first column and so on.
#' The total number of samples is  \code{sum(n)}
#' The distribution of  the first component is negative binomial with
#' parameters \code{mu1} and \code{phi1}. The second component follows
#' the same distribution outside gs. For the genes (rows) in gs the
#' distribution is negative binomial with parameters \code{(mu2,phi2)}.
#' A fraction \code{outlier.prop} of the pairs in \code{gs} follows
#' a negative binomial with parameters \code{(mu3,phi3)}
#' 
#' @param n Number of pairs
#' @param  N Number of genes
#' @param mu1 Mean of the first component
#' @param phi1 Dispersion of the first component
#' @param mu2 Mean of the second component
#' @param phi2 Dispersion of the second component
#' @param mu3 Mean of the second component
#' @param phi3 Dispersion of the second component
#' @param gs Significant gene set (a subset of rows in 1:N)
#' gs == NULL correspond with no differential expression
#' @param outlier.prop Proportion of pairs in \code{gs} with 
#' negative binomial distribution with parameters (mu3,phi3)
#' @export
#' @keywords 
#' Random counts 
#' 
pairedRandomCounts1 = function(n,N,mu1,phi1,mu2,phi2,mu3,phi3,
                               outlier.prop=.05,gs = NULL){   
    x0 = matrix(NA,nrow=N,ncol=2*n)
    group1 = seq(1,2*n-1,2)
    group2 = seq(2,2*n,2)
    outlier.prop = round(outlier.prop)
    x0[,group1] = MASS::rnegbin(N*n,mu=mu1,theta=phi1)
    if(!is.null(gs)){
    x0[gs,group2] = MASS::rnegbin(length(gs)*n,mu=mu2,theta=phi2)
    x0[gs,sample(group2,outlier.prop)] = 
      MASS::rnegbin(length(gs)*n,mu=mu3,theta=phi3)
    }
    if(!is.null(gs) & length(gs) < N)
        x0[-gs,group2] = MASS::rnegbin((N-length(gs))*n,mu=mu1,theta=phi1)
    if(is.null(gs))
        x0[,group2] = MASS::rnegbin(N*n,mu=mu1,theta=phi1)
    x0   
}


#' Random paired counts using Poisson distributions  
#' @description 
#' A gene expression matrix of counts with \code{N} features and
#' \code{length(n)} pairs of samples.
#' The first pair corresponds to the two first column and so on.
#' The total number of samples is  \code{2*n}
#' The distribution of  the first component is Poisson with
#' parameter \code{lambda1}. For the genes (rows) in
#' \code{gs} the distribution is the sum of the first generated value
#' plus a random count with Poisson distribution with mean \code{lambda2}.
#' For the genes (rows) not in  \code{gs} the distribution of the second component
#' is the sum of  the first generated value 
#' plus a random count with Poisson distribution with mean \code{lambda3}.
#' The matrix \code{outlier} contains in the first column the indices of rows and in
#' the second column the indices of pairs. For the pairs of counts in \code{outlier}
#' the first component follows a Poisson distribution with mean \code{lambda1}
#' and the second component is equal to the first one plus a random count with
#' Poisson distributed with mean \code{lambda4}.
#' 
#' @param n Number of pairs
#' @param  N Number of genes
#' @param lambda1 Mean of the first component
#' @param lambda2 Mean of the random count added to the first generated value
#' on the significant genes
#' @param lambda3 Mean of the random count added to the first generated value
#' on the non significant genes
#' @param lambda4 Mean of the random count added to the first generated value
#' on the outlier genes given in \code{outlier}
#' @param gs Significant gene set (a subset of rows in 1:N)
#' @param outlier Matrix (rows as genes and pairs as samples) where
#' the corresponding pair of counts are an outlier
#' gs == NULL correspond with no differential expression
#' 
#' @export
#' @keywords 
#' Random counts 
#'
rPairedPoisson = function(n,N,lambda1,lambda2,lambda3,lambda4,
                          gs = NULL,outlier=NULL){
    
    x0 = matrix(NA,nrow=N,ncol=2*n)
    group1 = seq(1,2*n-1,2)
    group2 = seq(2,2*n,2)
    x0[,group1] = matrix(stats::rpois(N*n,lambda=lambda1),nrow=N,ncol=n)
    if(!is.null(gs))
        x0[gs,group2] = matrix(stats::rpois(n*length(gs),lambda=lambda2),
                               nrow=length(gs),ncol=n)
    if(!is.null(gs) & length(gs) < N)
        x0[-gs,group2] = matrix(stats::rpois((N-length(gs))*n,lambda=lambda3),
                                nrow=N-length(gs),ncol=n)
    if(is.null(gs))
        x0[,group2] = matrix(stats::rpois(N*n,lambda=lambda3),nrow=N,ncol=n)
    if(!is.null(outlier)){
        outlier= as.matrix(outlier)
        x0[outlier[,1],2*outlier[,2]] =
            matrix(stats::rpois(nrow(outlier)*ncol(outlier),lambda=lambda4),
                   nrow=nrow(outlier),ncol=nrow(outlier))
    }

    x0[,group2] = x0[,group1] + x0[,group2]
    x0   
}
