 

###########################################################
#####  IDENTIFYING GENES ##################################
###########################################################

#' From id_name to ENTREZID, ENSEMBL and GENENAME
#' 
#' @description 
#' Given an ExpressionSet \code{eset} and an GeneData package 
#' \code{annotation_pkg} a data frame with identifiers id0 (the original 
#' identifier provides by the \code{eset}), a data frame is created with 
#' the original id0, and the corresponding ENTREZID, ENSEMBL and GENENAME
#'  
#' @param id_name Primary identifier
#' @param id Identifiers corresponding to id_name
#' @param annotation_pkg Annotation package
#' @param URL Should the url's been generated? 
#' @export
#' 
#'
#' 
#' 
tryannotation = function(id_name,
                         id,
                         annotation_pkg,
                         URL = TRUE){
  all_correspondences = 
    AnnotationDbi::select(annotation_pkg, keys=id,
                          column=c("ENTREZID","ENSEMBL","GENENAME"),          
                          keytype=id_name)
  sel = match(id,all_correspondences[,id_name])
  annot = all_correspondences[sel,]
  if(URL) annot[,"ENTREZID"] = entrezid2url(annot[,"ENTREZID"])
  if(URL) annot[,"ENSEMBL"] = ensembl2url(annot[,"ENSEMBL"])
  annot
}



#######################################################
################## URL's###############################
#######################################################

#' URL's from entrez identifiers
#' @description  
#' It makes the url's from entrez identifiers
#' @param id ENTREZ identifiers
#' @return 
#' The corresponding URL's in the database 
#' @export
#' @family URL generation
#' @seealso \link[tami]{ensembl2url}, \link[tami]{go2url}
entrezid2url = function(id)
  ifelse(id == "NA",NA,
         paste("<a href='http://www.ncbi.nlm.nih.gov/gene/?term=",
                            id,"'>",id,"</a>",sep=""))

#' URL's from ENSEMBL identifiers
#' @description  
#' It makes the url's from Ensembl identifiers. For details see 
#' \url{http://www.ensembl.org/info/docs/webcode/linking.html}
#' @param id ENSEMBL identifier(s)
#' @param site Site
#' @return 
#' The corresponding URL's in the database 
#' @export
#' @seealso \link[tami]{entrezid2url}, \link[tami]{go2url}
#' @family URL generation
ensembl2url = function(id,site="http://www.ensembl.org")
  paste("<a href='",site,"/id/",id,"'>",id, "</a>", sep="")


#' URL's from GO identifiers
#' @description  
#' It makes the url's from GO identifiers
#' @param id GO identifiers
#' @return 
#' The corresponding URL's in the database 
#' @export
#' @seealso \link[tami]{entrezid2url}, \link[tami]{ensembl2url}
#' @family URL generation

go2url = function(id)
  ifelse(id == "NA",NA,
         paste("<a href='http://amigo.geneontology.org/amigo/term/",
                        id,"'>",id,"</a>",sep=""))

#' URL's from KEGG identifiers
#' @description  
#' It makes the url's from KEGG identifiers
#' @param id KEGG identifiers
#' @export
#' @return 
#' The corresponding URL's in the database 
#' @seealso \link[tami]{entrezid2url}, \link[tami]{ensembl2url}
#' @family URL generation

kegg2url = function(id)
  ifelse(id == "NA",NA,
         paste("<a href='http://www.genome.jp/dbget-bin/www_bget?",
               id,"'>",id,"</a>",sep=""))

#' URL's from  C. elegans identifiers (WormBase)
#' @description  
#' It makes the url's from C. elegans identifiers
#' @param id C. elegans identifiers
#' @export
#' @return 
#' The corresponding URL's in the database 
#' @seealso \link[tami]{entrezid2url}, \link[tami]{ensembl2url},
#' \link[tami]{kegg2url}
#' @family URL generation

WormBase2url = function(id)
  ifelse(id == "NA",NA,
         paste("<a href='http://www.wormbase.org/species/c_elegans/gene/",
               id,"'>",id,"</a>",sep=""))
