#' Trapezoid rule for numerical integration 
#' @description 
#' An implementation of the trapezoid rule for numerical integration
#' @param x Abscisas
#' @param y Function values at \code{x}
#' 
TrapezoidRule = function(x,y){
  idx = 2:length(x)
  return (as.double( (x[idx] - x[idx-1]) %*% (y[idx] + y[idx-1])) / 2)
}

#' @importFrom  stats ecdf cov
NULL

#' Diggle's like randomization test
#' @description 
#' Diggle's like randomization test
#' @param x Matrix giving phenotype-expression associations obtained under
#' column randomizations. First column corresponds with real data.
#' @param gsc Gene set list where each element in the \code{list} gives us
#' the rows in the matrix \code{x} corresponding to the gene set 
#' @param method Distance used between the feature vector and the mean
#' of the features for the rest of \code{x} 
#' @export
#' 
digglet = function(x,gsc=list(1:nrow(x)),
                   method = c("Manhattan","Euclidean","Mahalanobis")){
  if(!is.list(gsc)) message("gsc should be a list")
  if(!is.matrix(x)) message("x should be a matrix")
  method = match.arg(method)
  a = pbapply::pbsapply(1:ncol(x),function(i) x[,i]- apply(x[,-i],1,mean)) - x
  if(method == "Manhattan")
    b = pbapply::pbsapply(gsc,function(rows) 
      apply(a[rows,],2,function(y)sum(abs(y))))
  if(method == "Euclidean") 
    b = sapply(gsc,function(rows) apply(a[rows,],2,function(y)sqrt(sum(y*y))))
  if(method == "Mahalanobis")
    b = pbapply::pbsapply(gsc,function(rows){
      invsigma0 = tryCatch(solve(cov(t(a[rows,]))),error = function(e){
                             cat("There is no inverse \n");diag(length(rows))})
      apply(a[rows,],2,function(y) t(y)%*%invsigma0%*%y)})
  
  apply(b,2,function(y) 1-ecdf(y[-1])(y[1]))
}


#' Diggle test for gene set collection using ecdf's
#' @description 
#' Given the gene set list \code{gsl} and the phenotype-expression
#' association \code{xt} where the first column, \code{xt[,1]}
#' corresponds to the original phenotype and the rest of columns
#' of \code{xt} (from the second to the last column) corresponds
#' to randomizated versions of the phenotipic variable 
#' @param gsl Gene set list 
#' @param xt Phenotype-expression association
#' @param nx Number of abscisas 
#' @param method Metric between cdf's
#' @export
#' @return 
#' A vector with names giving the names of \code{gsl} and
#' the p-values using the procedure described
diggle.cdf = function(gsl,xt,nx=100,method=c("Manhattan","Euclidean")){
  method = match.arg(method)
  aux.diggle.cdf = function(i,gsl0,xt0){
    xr = xt[gsl[[i]],]
    low = min(xr)
    up = max(xr)
    t0 = seq(low,up,length.out = nx)
    x = apply(xr,2,function(xc) ecdf(xc)(t0))
    a = sapply(1:ncol(x),function(i) x[,i]- apply(x[,-i],1,mean)) - x
    if(method == "Manhattan") b = apply(a,2,function(y)sum(abs(y)))
    if(method == "Euclidean") b = apply(a,2,function(y)sqrt(sum(y*y)))
    1-ecdf(b[-1])(b[1])
  }
  unlist(pbapply::pblapply(1:length(gsl),aux.diggle.cdf,gsl0=gsl,xt0 = xt))
}



