#' @importFrom  stats na.omit
NULL


#' Marginal gene testing
#' @description
#' The gene-phenotype association is quantified.
#' This value is evaluated for
#' each gene.
#' The randomization distribution of this gene-phenotype association is
#' obtained and the p-value corresponding to the observed values is returned.
#' @param x  Expression matrix (microarrays, RNA-seq,...)
#' @param y Two-level \code{factor} or its name
#' @param test The procedure used to test the differential
#' expression between the column groups of \code{x} defined by
#' \code{y}. Using this procedure the statitic or the p-value can be
#' used to quantify the gene-phenotype association.
#' @param gsc \code{List} with the gene set collection 
#' @param association We can work with \code{statistic} or \code{pvalue}
#' @param correction Method to correct for multiple comparisons
#' used by stats::p.adjust.
#' @param GeneNullDistr Gene null distribution 
#' \code{asymptotic}, \code{randomization} or \code{bootstrap} distribution
#' @param GeneSetNullDistr Gene set null distribution \code{self-contained} 
#' or \code{competitive}
#' @param alternative The alternative hypothesis to be tested
#' @param nmax Maximum number of randomizations or bootstrap resamples
#' @param id Gene identifiers (or the name) used in \code{gsc} (gene set 
#' collection)
#' @param minsize Minimum size for the gene set
#' @param maxsize Maximum size for the gene set
#' @param descriptive Function given the enrichment statistic per gene set
#' @param foutput File for html report
#' @param seed The seed for random number generation
#' @param tstat.return Is \code{tstat} returned?
#' @return
#' A matrix with the randomization distribution. The first column
#' corresponds to the original groups to be compared.
#' @export
#' 
GeneSetTest = function(x,y,test,gsc,
                       association=c("statistic","pvalue"),
                       correction=c("BH","bonferroni","holm","hochberg", 
                                    "hommel","BY","none"),
                       GeneNullDistr = c("randomization","asymptotic",
                                         "bootstrap"),
                       GeneSetNullDistr = c("self-contained","competitive"),
                       alternative=c("two-sided","less","greater"),
                       nmax = 100,
                       id = c("ENTREZID","ENSEMBL","GO","ORF"),
                       minsize=5,maxsize=100,
                       descriptive = c(mean,maxmean),
                       foutput="output",
                       seed=123,tstat.return=FALSE){
  association = match.arg(association)
  correction = match.arg(correction)
  GeneNullDistr = match.arg(GeneNullDistr)
  GeneSetNullDistr = match.arg(GeneSetNullDistr)
  alternative = match.arg(alternative)
  id = match.arg(id)
  ## Setting the seed 
  set.seed(seed)
  
  ## Is \code{id} the name or the variable?
  if(length(id) == 1){
    if(class(x)[1] == "ExpressionSet") id1 = Biobase::fData(x)[,id]
    if(class(x)[1] == "RangedSummarizedExperiment" | 
       class(x)[1]  == "SummarizedExperiment") 
      id1 = SummarizedExperiment::rowData(x)[,id]
  } else id1 = id
  
  ## Removing NA's 
  sel0 = which(!is.na(id1))
  x = x[sel0,]
  id1 = id1[sel0]
  
  ## Removing duplicities in id1 preserving first appearance
  sel1 = match(unique(id1),id1)
  x = x[sel1,]
  id1 = id1[sel1]
  
  ## Selecting gene set with size in [minsize,maxsize]
  ngs = sapply(gsc,length)
  sel2 = which( minsize <= ngs & ngs <= maxsize)
  gsc = gsc[sel2]
  
  ## Removing duplicities for genes not in selected gene sets
  sel3 = match(unique(unlist(gsc)),id1)
  sel3 = stats::na.omit(sel3)
  x = x[sel3,]
  id1 = id1[sel3]
  
  ## From identifiers to row indices for gene sets
  gsc1 = lapply(gsc,function(x0) which(is.element(id1,unlist(x0))))
  
  ## Selecting gene set with size in [minsize,maxsize]
  ngs = sapply(gsc1,length)
  sel2 = which( minsize <= ngs & ngs <= maxsize)
  gsc1 = gsc1[sel2]
  
  ## Generating ExpressionData  
  xed = convert(x,y)
  ny = table(groups(xed))
  n1 = ny[1]
  n = sum(ny)
  g1 = which(groups(xed) == levels(groups(xed))[1])
  x1 = cbind(exprm(xed)[,g1],exprm(xed)[,-g1]) 
  y1 = factor(rep(1:2,c(n1,n-n1)))
  if(GeneSetNullDistr == "asymptotic"){
    df = rowtest(x= exprm(xed),y=groups(xed),test,correction)
    if(association == "statistic") tstat = df[,"statistic"]
    if(association == "pvalue") tstat = df[,"rawp"]
  } 
  if(GeneSetNullDistr == "self-contained"){
    if(GeneNullDistr == "asymptotic") comb1 = matrix(1:n,ncol=1)
    if(GeneNullDistr == "randomization"){
      if(choose(n,n1) <= nmax) comb0  = combinat::combn(n,n1)
      else{
        comb0 = matrix(NA,nrow=n1,ncol=nmax)
        comb0[,1] = 1:n1
        for(i in 2:nmax) comb0[,i] = sample(n,n1)
      }
      comb1 = rbind(comb0,apply(comb0,2,setdiff,x=1:n))
    }
    if(GeneNullDistr == "bootstrap"){  
      comb1 = matrix(NA,nrow=n,ncol=nmax)
      comb1[,1] = 1:n
      for(i in 2:nmax) comb1[,i] = sample(n,replace = TRUE)
    }
    if(association == "statistic")
      tstat = pbapply::pbapply(comb1,2,function(sel) 
        test(x=x1[,sel],y=y1)[,"statistic"])
    if(association == "pvalue")
      tstat = pbapply::pbapply(comb1,2,function(sel) 
        test(x=x1[,sel],y=y1)[,"rawp"])
  }
  
  if(GeneSetNullDistr == "competitive"){
    if(association == "statistic")
      tstat0=rowtest(x= exprm(xed),y=groups(xed),test,correction)[,"statistic"]
    if(association == "pvalue")
      tstat0 = rowtest(x= exprm(xed),y=groups(xed),test,correction)[,"rawp"]
    if(GeneNullDistr == "randomization"){
      tstat = matrix(rep(tstat0,nmax),byrow = FALSE,nrow = nrow(exprm(xed)))
      tstat[,-1] = apply(tstat[,-1],2,sample)
    }
    if(GeneNullDistr == "bootstrap"){  
      tstat = matrix(rep(tstat0,nmax),byrow = FALSE,nrow = nrow(exprm(xed)))
      tstat[,-1] = apply(tstat[,-1],2,sample,replace = TRUE)
    }
  }
  
  tstatGeneSet = apply(tstat,2,function(xc) unlist(lapply(gsc1,function(x) descriptive(xc[x[[1]]]))))
  statistic = tstatGeneSet[,1]
  if(is.element(GeneSetNullDistr,c("self-contained","competitive"))){
    if(association == "pvalue") alternative = "less"
    rawp = randbootp(tstatGeneSet,alternative)
    adjp = stats::p.adjust(rawp,method = correction)
    df_results = data.frame(statistic,rawp,adjp)
  }
  ## Making output
  xf = new("GSAOutput",
           GeneSetData = data.frame(GO = names(gsc1)),
           GeneSetStat = df_results,
           GeneSetList = gsc1,
           correction = correction,
           foutput = foutput)
  ## Should tstat be returned?
  if(tstat.return) GeneStat(xf) = data.frame(tstat)
  xf
}

#' @title randbootp 
#' Randomization or bootstrap p-values
#' @description 
#' Given a matrix \code{x} of p-values obtained using the null distribution
#' given the data obtained sampling the randomization or bootstrap 
#' distribution are calculated.
#' @param x Matrix where each column corresponds with a randomization or a
#' bootstrap sample
#' @param alternative Alternative hypothesis \code{c("greater","less",
#' "two-sided")}
#' @return  
#' A vector with the p-value corresponding to each row
#' @export
randbootp = function(x,
                     alternative=c("two-sided","less","greater")){
  alternative = match.arg(alternative)
  if(alternative == "less")
    output = return(apply(x[,1] > x[,-1],1,sum)/(ncol(x)-1))
  if(alternative == "greater")
    return(apply(x[,1] < x[,-1],1,sum)/(ncol(x)-1))
  if(alternative == "two-sided"){
    pless    = apply(x[,1] > x[,-1],1,sum)/(ncol(x)-1)
    pgreater = apply(x[,1] < x[,-1],1,sum)/(ncol(x)-1)
    return(2*matrixStats::rowMins(cbind(pless,pgreater)))
  }
  NULL
}

#' Maximum of the means
#' @description 
#' The mean of the positive (respectively negative) values is calculated.
#' The maximum of the modulus of these values is returned
#' @param x Vector
#' @export
#' @return  
#' The maximum of the modulus of the means of positive and negative values
maxmean = function(x)
  max(mean(x[x>0],na.rm = TRUE),-mean(x[x<=0],na.rm = TRUE),na.rm=TRUE)



##################################################################
################# OVER-REPRESENTATION ANALYSIS ###################
##################################################################

#' @title ora
#' Over-representation analysis
#' @description 
#' Given a gene set \code{set0} and a list with a gene set collection,
#' an over-representation analysis is performed. 
#' Each set corresponds with a  row subset.  
#' @param set0 Significant genes
#' @param gsc List of gene set collection
#' @return A \code{data.frame} with the p-values, \code{rawp}, 
#' the observed odds ratio, \code{oddsratio}, and the confidence interval 
#' for the odds ratio, [\code{oddsratio_lower},\code{oddsratio_upper}]
#' @export
#' 
ora = function(set0,gsc){
  N = length(unique(unlist(gsc)))
  np1 = sapply(gsc,length)
  np2 = N - np1
  n1p = length(set0)
  n2p = N - n1p 
  n11 = sapply(gsc,function(set1,set0)sum(is.element(set1,set0)),set0)
  n21 = np1 - n11
  n12 = n1p - n11
  n22 = N - n11 - n12 - n21
  countmatrix = cbind(n11,n21,n12,n22)
  fisherRow = function(x){
    temp = stats::fisher.test(matrix(x,ncol=2,byrow = FALSE),
                              alternative="greater")
    c(temp$p.value,temp$estimate,temp$conf.int)
    }
  temp = t(pbapply::pbapply(countmatrix,1,fisherRow))
  result = data.frame(rawp = temp[,1],OR = temp[,2],
                      OR_low = temp[,3],OR_up  = temp[,4])
  result
}

#' Over-representation analysis 
#' @description 
#' Over-representation analysis. Given a significant gene set (or simply an 
#' important gene set) \code{SigGeneSet} the over-representation (a significant 
#' overlapping) of this \code{SigGeneSet} with a gene set collection is
#' tested using the unilateral Fisher test. If \code{SigGeneSet} is not
#' provided then \code{SigGeneSet} would be those genes with an adjusted
#' p-value lesser than \code{fdr}. 
#' @param x \code{DifferentialExpressionOutput} (if NULL the significant gene  
#' set is \code{SigGeneSet})
#' @param minsize Minimum size of the gene set
#' @param maxsize Maximum size of the gene set
#' @param correction Correction method for multiple comparisons
#' @param GeneSetList \code{list} of gene sets
#' @param SigGeneSet Significant gene set
#' @param foutput Output file for html report
#' @param id Type of identifier ("ENTREZID", "ENSEMBL") used in
#' \code{GeneSetList} and \code{SigGeneSet}
#' @export 
#' 
overRepresentation = function(x,minsize=5,maxsize=100,
                              correction = "BH",
                              GeneSetList = NULL,
                              SigGeneSet = NULL,
                              foutput="GSAout",id ="ENTREZID"){
  ## Is x a DifferentialExpressionOutput object? 
  if(class(x)[1] != "DifferentialExpressionOutput")
    return("x must be a DifferentialExpressionOutput")
  if(is.null(GeneSetList)) 
    return("A list of gene sets is required")
  correction = match.arg(correction)
  ngs = sapply(GeneSetList,length)
  sel0 = which(ngs >= minsize & ngs <= maxsize)
  if(length(sel0) == 0) return("There is no gene size in [minsize,maxsize]")
  GeneSetList0 = GeneSetList[sel0]
  if(is.null(SigGeneSet))
    SigGeneSet = GeneData(x)[GeneStat(x)[,"adjp"] < fdr(x),id]
  ora0 = ora(SigGeneSet,GeneSetList0)
  temp1 = rawp2adjpq(ora0[,"rawp"],correction)
  
  new("GSAOutput",
      GeneSetData=data.frame(GO = names(GeneSetList)[sel0]),
      GeneSetStat=data.frame(ora0,temp1),correction = correction,
      foutput =foutput)
}
