#' Generating sh file for STAR and Samtools
#' @description
#' Generation of shell file to generate the indexed genome,
#' to align the samples and to convert the sam to bam files
#' @param experiment Name of experiment
#' @param sraDir Directory with sra files
#' @param SRR_Acc_List Accession numbers in NCBI SRA
#' @param genomeDir Directory where to find the genome fastq file
#' @param fastqDir Directory with the fastq files
#' @param alignedDir Directory where the aligned sam files are saved
#' @param paired Is the RNA-Seq experiment paired ended or not?
#' @param steps steps[1] = 1 if the alignment step has to be generated
#' and 0 otherwise. steps[2] = 1 if the sam to sorted bam has to be
#' generated 
#' and 
#' @export
#' 
StarSamtools_sh  = function(experiment,sraDir = paste0(experiment,"/sra/"),
                          SRR_Acc_List="SRR_Acc_List",
                          genomeDir,
                          fastqDir = "fastq/",
                          alignedDir = "aligned/",
                          paired = TRUE,
                          steps=c(1,1)){
  file1 = paste0(experiment,"_alignment.sh")
  file2 = paste0(experiment,"_sam2bam.sh")
  if(steps[1] == 1 & paired){
    cat("#!/bin/bash \n",file = file1)
    for(name in SRR_Acc_List){
      name1 = paste0(fastqDir,name,"_1.fastq")
      name2 = paste0(fastqDir,name,"_2.fastq")
      toExecute = paste("STAR","--genomeDir",genomeDir,"--runThreadN",14,
                        "starIndex","--readFilesIn",name1,name2,
                        "--outFileNamePrefix",
                        paste0(alignedDir,name),"\n",sep=" ")
      cat(toExecute,file=file1,append=TRUE)
    }
  }
  
  if(steps[1] == 1 & !paired){
    cat("#!/bin/bash \n",file = file1)
    for(name in SRR_Acc_List){
      toExecute = paste("STAR","--genomeDir",genomeDir,"--runThreadN",14,
                        "starIndex","--readFilesIn",
                        paste0(fastqDir,name,".fastq"),
                        "--outFileNamePrefix",
                        paste0(alignedDir,name),"\n",sep=" ")
      cat(toExecute,file=file1,append=TRUE)
    }
  }
  

  if(steps[2] == 1 & paired){
    cat("#!/bin/bash \n",file = file2)
    
    for(name in SRR_Acc_List){
      toExecute = paste("samtools view -S -b",
                        paste0(alignedDir,name,"_1Aligned.out.sam"),
                        ">",paste0(alignedDir,name,"_1.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("samtools view -S -b",
                        paste0(alignedDir,name,"_2Aligned.out.sam"),
                        ">",paste0(alignedDir,name,"_2.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm",paste0(alignedDir,name,"_1Aligned.out.sam"),"\n",
                        sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm",paste0(alignedDir,name,"_2Aligned.out.sam"),"\n",
                        sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("samtools sort",
                        paste0(alignedDir,name,"_1.bam"),
                        "-o",paste0(alignedDir,name,"_1_sort.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("samtools sort",
                        paste0(alignedDir,name,"_2.bam"),
                        "-o",paste0(alignedDir,name,"_2_sort.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm ",paste0(alignedDir,name,"_1.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm ",paste0(alignedDir,name,"_2.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
    }
  }

  if(steps[2] == 1 & !paired){
    cat("#!/bin/bash \n",file = file2)
    
    for(name in SRR_Acc_List){
      toExecute = paste("samtools view -S -b",
                        paste0(alignedDir,name,"Aligned.out.sam"),
                        ">",paste0(alignedDir,name,".bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm",paste0(alignedDir,name,"Aligned.out.sam"),"\n",
                        sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("samtools sort",
                        paste0(alignedDir,name,".bam"),
                        "-o",paste0(alignedDir,name,"_sort.bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
      toExecute = paste("rm ",paste0(alignedDir,name,".bam"),"\n",sep=" ")
      cat(toExecute,file=file2,append=TRUE)
    }
  }
}  


